@@ -8,9 +8,8 @@ use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:MAROS';
-our $VERSION = '1.15';
+our $VERSION = '1.16';
 
-use List::Util qw(max);
 use MooseX::App::Meta::Role::Attribute::Option;
 use MooseX::App::Exporter qw(app_base app_fuzzy option);
 use MooseX::App::Message::Envelope;
@@ -43,6 +42,7 @@ sub init_meta {
     $args{roles}        = ['MooseX::App::Role::Base'];
     $args{metaroles}    = {
         class               => ['MooseX::App::Meta::Role::Class::Base'],
+        attribute           => ['MooseX::App::Meta::Role::Attribute::Option'],
     };
     
     return MooseX::App::Exporter->process_init_meta(%args);
@@ -80,9 +80,10 @@ sub new_with_command {
         Moose->throw_error('new_with_command got inavlid extra arguments');
     }
     
-    # Localize and encode @ARGV
-    local @ARGV = MooseX::App::Utils::encoded_argv();
-    my $first_argv = shift(@ARGV);
+    # Get ARGV
+    my $parsed_argv = MooseX::App::ParsedArgv->new;
+    $parsed_argv->argv(\@ARGV);
+    my $first_argv = $parsed_argv->shift_argv;
     
     # No args
     if (! defined $first_argv
@@ -90,7 +91,7 @@ sub new_with_command {
         || $first_argv =~ m/^-/) {
         return MooseX::App::Message::Envelope->new(
             $meta->command_message(
-                header          => "Missing command",
+                header          => "Missing command", # LOCALIZE
                 type            => "error",
             ),
             $meta->command_usage_global(),
@@ -102,7 +103,7 @@ sub new_with_command {
         );
     # Looks like a command
     } else {
-        my $return = $meta->command_get($first_argv);
+        my $return = $meta->command_find($first_argv);
         
         # Nothing found
         if (blessed $return
@@ -113,7 +114,7 @@ sub new_with_command {
             );
         # One command found
         } else {
-            my $command_class = $meta->app_commands->{$return};
+            my $command_class = $meta->command_get($return);
             return $class->initialize_command_class($command_class,%args);
         }
     }
@@ -149,7 +150,7 @@ In your base class:
   ); # not exposed
 
 Write multiple command classes (If you have only a single command class
-you might use L<MooseX::App::Simple> instead)
+you should use L<MooseX::App::Simple> instead)
 
   package MyApp::SomeCommand;
   use MooseX::App::Command; # important
@@ -158,6 +159,7 @@ you might use L<MooseX::App::Simple> instead)
   option 'some_option' => (
       is            => 'rw',
       isa           => 'Str',
+      required      => 1,
       documentation => q[Very important option!],
   );
   
@@ -172,10 +174,28 @@ And then in some simple wrapper script:
  use MyApp;
  MyApp->new_with_command->run;
 
+On the command line:
+
+ bash$ myapp some_command --help
+ usage:
+     myapp some_command [long options...]
+     myapp help
+     myapp some_command --help
+ 
+ global options:
+     --global_option    Enable this to do fancy stuff [Flag]
+     --some_option      Very important option! [Required]
+     --help --usage -?  Prints this usage information. [Flag]
+ 
+ available commands:
+     some_command    Description of some command
+     another_command Description of another command
+     help            Prints this usage information
+
 =head1 DESCRIPTION
 
 MooseX-App is a highly customizeable helper to write user-friendly 
-command-line applications without having to worry about most of the annoying 
+command line applications without having to worry about most of the annoying 
 things usually involved. Just take any existing L<Moose> class, add a single 
 line (C<use MooseX-App qw(PluginA PluginB ...);>) and create one class
 for each command in an underlying namespace.
@@ -188,7 +208,9 @@ MooseX-App will then take care of
 
 =item * Creating automated help and doucumentation from pod and attributes
 
-=item * Reading and validating the command line options entered by the user
+=item * Reading, encoding and validating the command line options entered by the user
+
+=item * Providing helpful error messages if user input cannot be validated
 
 =back
 
@@ -206,6 +228,7 @@ This is equivalent to
       is            => 'rw',
       isa           => 'Str',
       traits        => ['AppOption'],
+      cmd_option    => 1,
   );
 
 Read the L<Tutorial|MooseX::App::Tutorial> for getting started with a simple 
@@ -223,7 +246,7 @@ object holding an error message.
 
 You can pass a hash of default params to new_with_command
 
- MyApp->new_with_command( %default );
+ MyApp->new_with_command(%default);
 
 =head2 initialize_command_class
 
@@ -282,7 +305,9 @@ documentation on how to create your own plugins.
 Read the L<Tutorial|MooseX::App::Tutorial> for getting started with a simple 
 MooseX::App command line application.
 
-L<MooseX::App::Cmd>, L<MooseX::Getopt> and L<App::Cmd>
+For alternatives you can check out
+
+L<MooseX::App::Cmd>, L<MooseX::Getopt>, L<MooX::Options> and L<App::Cmd>
 
 =head1 SUPPORT
 
@@ -303,7 +328,7 @@ your report as I make changes.
 =head1 CONTRIBUTORS
 
 In no particular order: Andrew Jones, George Hartzell, Steve Nolte, 
-Michael G, Thomas Klausner, Yanick Champoux
+Michael G, Thomas Klausner, Yanick Champoux, Edward Baudrez
 
 =head1 COPYRIGHT
 
